/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of the Trojita Qt IMAP e-mail client,
   http://trojita.flaska.net/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import "../Utils/Utils.js" as Utils

// TODO: this need massively reworking as nothing is lining up correctly
// What was i thinking???
ListItem.Expandable {
    id: headerExpandable

    // Properties
    height: expanded ? undefined : units.gu(8)
    anchors {
        top: parent.top
        left: parent.left
        right: parent.right
//        leftMargin: units.gu(1)
        rightMargin: units.gu(1)
        topMargin: units.gu(1.5)
    }
//            showDivider: false
    expandedHeight: infoColumn.height + units.gu(2)

    property alias fromAddress: fromAddressWidget.text

    // Components
    UbuntuShape {
        id: msgListContactImg
        height: units.gu(5.5)
        width: height
        color: Utils.getIconColor(fromNameWidget.text)
        anchors {
            top: parent.top
            left: parent.left
        }
        Image {
            anchors.fill: parent
            anchors.margins: units.gu(0.5)
            source: Qt.resolvedUrl("../../icons/contact.svg")
        }
    }
    Image {
        id: conIcon
        anchors {
            leftMargin: units.gu(1)
            left: msgListContactImg.right
            top: msgListContactImg.top
        }
        height: units.gu(2)
        width: height
        source: Qt.resolvedUrl("../../icons/contact_grey.svg")
        MouseArea {
            anchors.fill: parent
            onClicked: contactNameClicked(imapAccess.oneMessageModel.from)
        }
    }
    Image {
        anchors {
            leftMargin: units.gu(1)
            topMargin: units.gu(1)
            left: msgListContactImg.right
            top: conIcon.bottom
        }
        height: units.gu(2)
        width: height
        source: Qt.resolvedUrl("../../icons/email.svg")
    }

    Column {
        id: infoColumn
        property real contentLeftMargin: msgListContactImg.width + conIcon.width + units.gu(2)
        width: parent.width
        spacing: units.gu(0.5)

        AddressWidget {
            id: fromNameWidget
            anchors {
                left: parent.left
                leftMargin: infoColumn.contentLeftMargin
            }
            color: UbuntuColors.midAubergine
            address: imapAccess.oneMessageModel.from[0]
            nameOnly: true
        }
        AddressWidget {
            id: fromAddressWidget
            anchors {
                left: parent.left
                leftMargin: infoColumn.contentLeftMargin
            }
            address: imapAccess.oneMessageModel.from[0]
            addressOnly: true
        }
        Label {
            id: dateLabel
            anchors {
                left: parent.left
                leftMargin: infoColumn.contentLeftMargin
                right: parent.right
            }
            fontSize: "small"
            text: i18n.tr(Utils.formatDateDetailed(imapAccess.oneMessageModel.date))
        }
        Item {
            width: parent.width
            height: toWidget.height
            visible: headerExpandable.expanded ? toWidget.address.length > 0 : false
            Label {
                id: toCaptionLabel
                x: infoColumn.contentLeftMargin - width - units.gu(1)
                text: i18n.tr("To:")
                font.bold: true
                horizontalAlignment: Text.AlignRight
                elide: Text.ElideRight
            }
            AddressWidgets {
                id: toWidget
                anchors {
                    left: toCaptionLabel.right
                    leftMargin: units.gu(1)
                    right: parent.right
                }
                address: imapAccess.oneMessageModel.to
            }
        }
        Item {
            visible: headerExpandable.expanded ? ccWidget.address.length > 0 : false
            width: parent.width
            height: ccWidget.height
            Label {
                id: ccCaptionLabel
                x: infoColumn.contentLeftMargin - width - units.gu(1)
                text: i18n.tr("Cc:")
                font.bold: true
                horizontalAlignment: Text.AlignRight
                elide: Text.ElideRight
            }
            AddressWidgets {
                id: ccWidget
                anchors {
                    left: ccCaptionLabel.right
                    leftMargin: units.gu(1)
                    right: parent.right
                }
                address: imapAccess.oneMessageModel.cc
            }
        }
        Item {
            visible: headerExpandable.expanded ? bccWidget.address.length > 0 : false
            width: parent.width
            height: bccWidget.height
            Label {
                id: bccCaptionLabel
                x: infoColumn.contentLeftMargin - width - units.gu(1)
                text: i18n.tr("BCc:")
                font.bold: true
                horizontalAlignment: Text.AlignRight
                elide: Text.ElideRight
            }
            AddressWidgets {
                id: bccWidget
                anchors {
                    left: bccCaptionLabel.right
                    leftMargin: units.gu(1)
                    right: parent.right
                }
                address: imapAccess.oneMessageModel.bcc
            }
        }
        Item {
            width: parent.width
            height: subjectLabel.height
            visible: headerExpandable.expanded ? toWidget.address.length : false
            Label {
                id: subjectLabelLabel
                x: infoColumn.contentLeftMargin - width - units.gu(1)
                text: i18n.tr("Subject:")
                font.bold: true
                horizontalAlignment: Text.AlignRight
                elide: Text.ElideRight
            }
            Label {
                id: subjectLabel
                anchors {
                    left: subjectLabelLabel.right
                    leftMargin: units.gu(1)
                    right: parent.right
                }
                text: imapAccess.oneMessageModel.subject
                wrapMode: Text.WordWrap
            }
        }
    }
    Image {
        id: dropDownIcon
        anchors {
            right: parent.right
            top: parent.top
            topMargin: units.gu(1)
            rightMargin: units.gu(1)
        }
        height: units.gu(3)
        width: height
        source: Qt.resolvedUrl("../../icons/dropdown-menu.svg")
        rotation: headerExpandable.expanded ? 180 : 0
        MouseArea {
            anchors.fill: parent
            onClicked: {
                headerExpandable.expanded = !headerExpandable.expanded
            }
        }
    }
}

