/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of Dekko email client for Ubuntu Touch/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.2
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import "../Components"


ListItemWithActions {
    id: mailboxDelegate

    property bool isFirst: false
    property bool active: false
    showDivider: true

    signal openChildMailbox(var index, string shortMboxName, string mboxName)

    height: units.gu(8)
    color: Theme.palette.normal.background
    triggerActionOnMouseRelease: true

    function getIconSourceForMailbox() {
        if (mailboxIsINBOX) {
            return Qt.resolvedUrl("../../icons/actions/inbox.svg")
        } else if (isGmailImportantSpecialUse) {
            return Qt.resolvedUrl("../../icons/actions/important.svg")
        } else if (mailboxIsSpecialUseALL) {
            return Qt.resolvedUrl("../../icons/actions/inbox.svg")
        } else if (mailboxIsSpecialUseFLAGGED) {
            return Qt.resolvedUrl("../../icons/actions/flagged.svg")
        } else if (mailboxIsSpecialUseSENT || mailboxIsSENT) {
            return Qt.resolvedUrl("../../icons/actions/send.svg")
        } else if (mailboxIsSpecialUseTRASH || mailboxIsTRASH) {
            return Qt.resolvedUrl("../../icons/actions/trash.svg")
        } else if (mailboxIsSpecialUseJUNK || mailboxIsJUNK) {
            return Qt.resolvedUrl("../../icons/actions/spam.svg")
        } else if (mailboxIsSpecialUseDRAFTS || mailboxIsDRAFTS) {
            return Qt.resolvedUrl("../../icons/actions/edit.svg")
        } else {
            return Qt.resolvedUrl("../../icons/actions/folder.svg")
        }
    }

    transitions: [
        Transition {
            UbuntuNumberAnimation {
                properties: "opacity"
            }
        }
    ]

    Rectangle {
        anchors {
            fill: parent
            topMargin: units.gu(-1)
            bottomMargin: units.gu(-1)
            leftMargin: units.gu(-2)
            rightMargin: units.gu(-2)
        }
        opacity: mailboxDelegate.active ? 0.2 : 0.0
        color: "black"
        Behavior on opacity {
            UbuntuNumberAnimation {}
        }
    }

    Item {
        anchors.fill: parent
        anchors.margins: units.gu(0.2)
        Icon {
            id: accountsImage
            height: units.gu(4); width: height
            anchors {
                left: parent.left
                top: parent.top
                topMargin: units.gu(0.5)
            }
            source: getIconSourceForMailbox()
        }

        Label {
            id: nameLabel
            anchors {
                left: accountsImage.right
                leftMargin: units.gu(1)
                top: parent.top
                topMargin: units.gu(1.2)
                right: progression.left
            }
            text: shortMailboxName
            fontSize: "medium"
            clip: true
            color: UbuntuColors.lightAubergine
        }

        Label {
            id: unreadLabel
            anchors {
                right: mailboxHasChildMailboxes ? progression.left : parent.right
                rightMargin: units.gu(1)
                top: parent.top
                topMargin: units.gu(0.4)
            }
            visible: mailboxIsSelectable
            text: i18n.tr("Unread: %1").arg(unreadMessageCount)
            fontSize: "small"
            clip: true
            color: UbuntuColors.lightAubergine
        }

        Label {
            anchors {
                top: unreadLabel.bottom
                right: mailboxHasChildMailboxes ? progression.left : parent.right
                rightMargin: units.gu(1)
            }
            visible: mailboxIsSelectable
            text: i18n.tr("Total: %1").arg(totalMessageCount)
            fontSize: "small"
            clip: true
        }

        Icon {
            id: progression
            visible: mailboxHasChildMailboxes
            anchors {
                right: parent.right
                top: parent.top
                topMargin: units.gu(1.5)
                bottom: parent.bottom
                bottomMargin: units.gu(1.5)
            }
            width: units.gu(2)
            name: "go-next"

            MouseArea {
                anchors.centerIn: parent
                height: parent.height
                width: units.gu(4)
                onReleased: openChildMailbox(index, shortMailboxName, mailboxName)
            }
        }
    }
}
