/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of the Dekko email client for Ubuntu Touch/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.2
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItems
import "../Utils/Utils.js" as Utils
import TrojitaCore 0.1
import DekkoCore 0.1
import "../Components"


ListItemWithActions {
    id: normalMessageItemDelegate

    //---------------------------------
    // PROPERTIES
    //---------------------------------
    property bool isFirst: false
    property bool active: false
    //---------------------------------
    // SIGNALS
    //---------------------------------
    signal markMessageDeleted()
    signal markMessageFlagged()

    //---------------------------------
    // FUNCTIONS
    //---------------------------------

    //---------------------------------
    // OBJECT PROPERTIES
    //---------------------------------
    showDivider: true
    height: units.gu(8)
    color: Theme.palette.normal.background
    triggerActionOnMouseRelease: true

    //---------------------------------
    // ACTIONS
    //---------------------------------

    function handleMarkMessageForDeletionByUserPrefs() {
        // eitherway we need to emit the markMessageDeleted signal so
        // lets do that first
        markMessageDeleted()
        // Now the user has the option to auto expunge on delete
        // at the moment this has a nasty side affect of resetting the listview
        // to the top, but should be fixed in Qt 5.3.1 so we will keep it here
        // for when that arrives
        if (currentAccount.imapAutoExpungeOnDelete) {
            imapAccess.imapModel.expungeMailbox(imapAccess.msgListModel.currentMailbox())
        } else {
            // so we only want to mark the message as deleted. In the future we
            // should probably provide a move to trash action. But we will stick with the
            // strikethrough for now
            return;
        }
    }

    //---------------------------------
    // COMPONENTS
    //---------------------------------
    Rectangle {
        anchors {
            fill: parent
            topMargin: units.gu(-1)
            bottomMargin: units.gu(-1)
            leftMargin: units.gu(-2)
            rightMargin: units.gu(-2)
        }
        opacity: isMarkedRead ? 0.0 : 1.0
        color: "#ffffff"
        Behavior on opacity {
            UbuntuNumberAnimation {}
        }
    }

    Item {
        id: main
        anchors { fill: parent; margins: units.gu(0.2) }

        Item {
            id: infoDelegate
            anchors.fill: parent
            anchors.topMargin: units.gu(0.2)
            visible: isFetched

            UbuntuShape {
                id: msgListContactImg
                height: units.gu(5)

                width: height
                radius: "medium"
                color: Utils.getIconColor(fromLabel.text)
                anchors {
                    top: parent.top
                    //                margins: units.gu(0.5)
                    left: parent.left
                }
                Image {
                    anchors.fill: parent
                    anchors.margins: units.gu(0.5)
                    source: Qt.resolvedUrl("../../icons/contact.svg")
                }
            }

            Label {
                id: fromLabel
                anchors {
                    left: msgListContactImg.right
                    leftMargin: units.gu(1)
                    top: parent.top
                    //                topMargin: units.gu(1.2)

                    right: favTimColumn.left
                    rightMargin: units.gu(0.5)
                }
                text: from ? Utils.formatMailAddresses(from, true) : ""
                fontSize: "medium"
                font.strikeout: isMarkedDeleted
                font.weight: isMarkedRead ? Font.Light : Font.DemiBold
                clip: true
                color: UbuntuColors.lightAubergine
            }

            Label {
                id: subjectLabel
                anchors {
                    left: msgListContactImg.right
                    leftMargin: units.gu(1)
                    top: fromLabel.bottom
                    topMargin: units.gu(0.4)
                    right: favTimColumn.left
                    rightMargin: units.gu(0.5)
                }
                text: subject ? subject : i18n.tr('<No Subject>')
                fontSize: "small"
                clip: true
                font.strikeout: isMarkedDeleted
                font.weight: isMarkedRead ? Font.Light : Font.DemiBold
            }
            Column {
                id: favTimColumn
                spacing: units.gu(0.5)
                width: timeLabel.width
                anchors {
                    right: parent.right
                    leftMargin: units.gu(0.5)
                    top: parent.top
                }
                Label{
                    id: timeLabel
                    text: model ? Utils.prettyDate(model.date): "";
                    font.pixelSize: FontUtils.sizeToPixels("small")
                }

                Row{
                    height: units.gu(2)
                    spacing: units.gu(1)
                    Image {
                        visible: isFetched ? hasAttachments : null
                        height: units.gu(2)
                        width: height
                        source: Qt.resolvedUrl("../../icons/attach.svg")
                    }
                    Image {
                        height: units.gu(2)
                        width: height
                        source: (isMarkedFlagged) ? Qt.resolvedUrl("../../icons/favorite-selected.svg") : Qt.resolvedUrl("../../icons/favorite-unselected.svg")
                        MouseArea {
                            anchors.fill: parent
                            onClicked: markMessageFlagged()
                        }
                    }
                    anchors.right: parent.right

                }
            }
        }

        ListItems.Standard {
            id: loadingIndicator
            height: parent.height
            text: isUnavailable ? i18n.tr("Unavailable") : i18n.tr("Loading...")
            //read model.subject to trigger fetch
            visible: !isFetched
            showDivider: false
        }
    }

    //---------------------------------
    // TRANSITIONS
    //---------------------------------
    transitions: [
        Transition {
            UbuntuNumberAnimation {
                properties: "opacity"
            }
        }
    ]
}
