/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of Dekko email client for Ubuntu Devices/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.2
import Ubuntu.Components 1.1
import Ubuntu.Components.Themes.Ambiance 0.1
import DekkoCore 0.1
import "../Components"

Item {
    id: plainTextPart
    height: plainView.height
    width: parent.width

    property string fullMessage
    property alias text: plainView.text
    property alias msgPartIndex: plainTextFormatter.msgPartIndex
    readonly property alias selectedText: plainView.selectedText

    PlainTextFormatter {
        id: plainTextFormatter
        msgIndex: imapAccess.oneMessageModel.messageIndex
        format: PlainTextFormatter.LINKIFIED
        onMessageChanged: {
            plainTextPart.text = message
            fullMessage = rawMessage
        }
    }

    TextArea {
        id: plainView
        anchors {
            left: parent.left
            top: parent.top
            right: parent.right
            topMargin: units.gu(2)
        }
        text: plainTextFormatter.message
        autoSize: true
        maximumLineCount: 0
        cursorVisible: false
        readOnly: true
        wrapMode: TextEdit.WrapAtWordBoundaryOrAnywhere
        textFormat: TextEdit.RichText
        // Override the default style and make it 'look' like we have loaded it in a webview
        // but we will be able to highlight specific text parts for the reply
        // if the user doesn't want to include it all.
        style:TextAreaStyle {
            overlaySpacing: 0
            frameSpacing: 0
            background: Item {}
        }
        onLinkActivated: Qt.openUrlExternally(link)
    }

    Binding {
        target: oneMessagePage
        property: "replyMessage"
        value: plainTextPart.selectedText ? plainTextPart.selectedText : fullMessage
    }
}
