/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of Dekko email client for Ubuntu Devices/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import DekkoCore 0.1
import "../Composer"

UbuntuListView {
    id: inputListView

    property alias nameField: nameField.text
    property alias emailField: emailField.text
    property alias organizationField: organizationField.text
    property alias signatureField: signatureField.text

    function save() {
        console.log("Build new identity")
        newIdentity.name = nameField.text
        newIdentity.email = emailField.text
        newIdentity.organization = organizationField.text
        newIdentity.signature = signatureField.text
        currentAccount.identityModel.appendRow(newIdentity)
    }

    anchors.fill: parent
    model: composerItemModel
    clip: true

    SenderIdentity {
        id: newIdentity
        useDefaultSignature: true
    }

    VisualItemModel {
        id: composerItemModel
        ListItem.Header {
            text: i18n.tr("Name (Required)")
        }
        SenderIdentityInputField{
            id: nameField
            width: inputListView.width
            placeholderText: i18n.tr("Enter Name")
        }
        ListItem.Header {
            text: i18n.tr("Email Address (Required)")
        }
        SenderIdentityInputField{
            id: emailField
            width: inputListView.width
            inputMethodHint: Qt.ImhEmailCharactersOnly | Qt.ImhNoAutoUppercase
            placeholderText: i18n.tr("Enter Email Address")
        }
        ListItem.Header {
            text: i18n.tr("Organization")
        }
        SenderIdentityInputField{
            id: organizationField
            width: inputListView.width
            placeholderText: i18n.tr("Enter Organization")
        }
        ListItem.Header {
            text: i18n.tr("Signature")
        }
        MessageField {
            id: signatureField
            width: inputListView.width
            placeholderText: i18n.tr("Enter Signature")
            text: newIdentity.signature
            onHasActiveFocusChanged: {
                if (hasActiveFocus) {
                    positionViewAtIndex(8, ListView.Beginning)
                } else {
                    console.log("Message field lost active focus")
                }
            }
            // start with an initial negative value to be sure key events trigger from the off,
            // as the cursor's y position if text area is empty will be 0.0.
            property real positionY: -1.0
            // lets try our best to keep the cursor in the View
            // as we have autosize enabled and positionViewAtXX doesn't provide the means
            // for mid message editing without the cursor going off screen.
            // This quite a simple solution at the moment where we just use
            // key release events, as this is the only time we should update/re-position the view,
            // in case the user is moving around mid message, using cursorposition changed events
            // would cause the view to skip all over the place.
            onKeyPressed: {
                var p = getRectPos().y
                if (p === positionY) {
                    return
                } else {
                    inputListView.contentY = contentY + p
                }
                positionY = p
                inputListView.returnToBounds()
            }
        }
        // empty item to position view to on active focus
        // if messageField visible area is greater than flickable visible area
        Item {
            height: units.gu(5)
            width: inputListView.width
        }
    }
}
