/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of the Trojita Qt IMAP e-mail client,
   http://trojita.flaska.net/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItems
import Ubuntu.Components.Popups 1.0
import TrojitaCore 0.1
import DekkoCore 0.1

Flickable {
    anchors.fill: parent
    clip: true
    contentHeight: col.height + units.gu(5)
    id: smtpAccountSettingsPage

    property Account account
    property alias smtpUserName: smtpUserNameInput.text
    property alias smtpServer: smtpServerInput.text
    property alias smtpPort: smtpPortInput.text
    property alias smtpPassword: smtpPasswordInput.text
    property alias smtpAuthenticate: authCheckBox.checked
    property alias smtpBurl: burlCheckbox.checked
    property alias smtpSaveToImap: outgoingCheckbox.checked
    property alias smtpSentLocation: smtpSentLocationInput.text
    property alias smtpPathToSendmail: smtpSendMailInput.text
    property alias smtpConnectionModelIndex: connectionSelector.selectedIndex
    // FIXME: default to encryption, similar to how the DesktopGui does that
    property alias smtpEncryptionModelIndex: encryptionSelector.selectedIndex
    //TODO: what about sendmail and imapsendmail required values??
    property bool settingsValid: {
        if (smtpConnectionModelIndex === 0) {
            if (smtpAuthenticate) {
                return smtpServerLabel.valid && smtpUsernameLabel.valid
            } else {
                return smtpServerLabel.valid
            }
        } else {
            return true
        }
    }

    function saveSettings() {
        console.log("[LOG] Saving SMTP settings...............")
        account.submissionUsername = smtpUserName
        account.submissionServer = smtpServerInput.text
        account.submissionAuthenticateEnabled = smtpAuthenticate
        account.submissionUseBurl = smtpBurl
        account.submissionSaveToImap = smtpSaveToImap
        account.submissionSentMailboxName = smtpSentLocationInput.text
        account.submissionPort = smtpPortInput.text
        console.log("[LOG] SMTP settings saved successfully")
    }

    onSmtpConnectionModelIndexChanged: {

        switch (connectionSelector.model.get(connectionSelector.selectedIndex).name) {
        case "NETWORK":
            smtpAccountSettingsPage.state = "NETWORK_SETTINGS"
            account.submissionMethod = encryptionSelector.model.get(encryptionSelector.selectedIndex).method
            break
        case "SENDMAIL":
            smtpAccountSettingsPage.state = "SENDMAIL_SETTINGS"
            account.submissionMethod = SubmissionMethod.SENDMAIL
            break
        case "IMAP_SENDMAIL":
            smtpAccountSettingsPage.state = "IMAP_SENDMAIL_SETTINGS"
            account.submissionMethod = SubmissionMethod.IMAP_SENDMAIL
            break
        }
    }

    onSmtpEncryptionModelIndexChanged: {
        account.submissionMethod = encryptionSelector.model.get(encryptionSelector.selectedIndex).method
        smtpPortInput.text = encryptionSelector.model.get(encryptionSelector.selectedIndex).port

    }

    states: [
        State {
            name: "NETWORK_SETTINGS"
            PropertyChanges { target: smtpAccountSettingsPage; smtpConnectionModelIndex: 0 }
            PropertyChanges { target: smtpServerLabel; visible: true }
            PropertyChanges { target: smtpPortLabel; visible: !smtpPortWarnLabel.visible }
            PropertyChanges { target: authItem; visible: true }
        },
        State {
            name: "SENDMAIL_SETTINGS"
            PropertyChanges { target: smtpAccountSettingsPage; smtpConnectionModelIndex: 1 }
            PropertyChanges { target: smtpUsernameLabel; visible: false }
            PropertyChanges { target: smtpServerLabel; visible: false }
            PropertyChanges { target: smtpPortLabel; visible: false }
            PropertyChanges { target: smtpPortWarnLabel; visible: false }
            PropertyChanges { target: smtpSendMail; visible: true }
            PropertyChanges { target: authItem; visible: false }
            PropertyChanges { target: authItem; checked: false }
            PropertyChanges { target: burlItem; visible: false }
        },
        State {
            name: "IMAP_SENDMAIL_SETTINGS"
            PropertyChanges { target: smtpAccountSettingsPage; smtpConnectionModelIndex: 2 }
            PropertyChanges { target: smtpUsernameLabel; visible: false }
            PropertyChanges { target: smtpServerLabel; visible: false }
            PropertyChanges { target: smtpPortLabel; visible: false }
            PropertyChanges { target: smtpPortWarnLabel; visible: false }
            PropertyChanges { target: smtpSendMail; visible: false }
            PropertyChanges { target: authItem; visible: false }
            PropertyChanges { target: authItem; checked: false }
            PropertyChanges { target: burlItem; visible: false }
        }
    ]


    Column {
        id: col
        spacing: units.gu(1)
        anchors {
            top: parent.top
            left: parent.left
            right: parent.right
            margins: units.gu(2)
        }

        OptionSelector {
            id: connectionSelector
            text: i18n.tr("Connection Method")
            model: connectionMethodModel
            // Disabling this for now as we don't currently support IMAP sendmail or sendmail
            visible: false
            showDivider: false
            expanded: false
            colourImage: true
            delegate: connectionMethodDelegate
        }
        Component {
            id: connectionMethodDelegate
            OptionSelectorDelegate { text: description }
        }
        ListModel {
            id: connectionMethodModel
            function init() {
                connectionMethodModel.append({ "name": "NETWORK", "description": i18n.tr("Network") })
                connectionMethodModel.append({ "name": "SENDMAIL", "description": i18n.tr("Local sendmail-compatible") })
                connectionMethodModel.append({ "name": "IMAP_SENDMAIL", "description": i18n.tr("IMAP SENDMAIL Extension") })
            }
            Component.onCompleted: init()
        }

        OptionSelector {
            id: encryptionSelector
            text: i18n.tr("Secure Connection")
            visible: connectionSelector.selectedIndex === 0
            model: encryptionMethodModel
            showDivider: false
            expanded: false
            colourImage: true
            delegate: encryptionMethodDelegate
        }
        Component {
            id: encryptionMethodDelegate
            OptionSelectorDelegate { text: description }
        }
        ListModel {
            id: encryptionMethodModel
        }

        Label {
            id: smtpServerLabel
            property bool valid: !!smtpServerInput.text
            text: valid ? i18n.tr("Server address") : i18n.tr("Server address (required)")
            color: valid ? "#888888" : "red"
        }
        TextField {
            id: smtpServerInput
            visible: smtpServerLabel.visible
            inputMethodHints: Qt.ImhNoAutoUppercase | Qt.ImhPreferLowercase | Qt.ImhUrlCharactersOnly | Qt.ImhNoPredictiveText
            anchors {
                left: col.left
                right: col.right
            }
        }

        Label {
            id: smtpPortLabel
            visible: true
            text: i18n.tr("Port")
        }
        Label {
            id: smtpPortWarnLabel
            color: "red"
            visible: smtpPortWarnLabel.text
        }
        TextField {
            id: smtpPortInput
            visible: smtpPortLabel.visible || smtpPortWarnLabel.visible
            inputMethodHints: Qt.ImhDigitsOnly
            validator: IntValidator { bottom: 1; top: 65535 }
            anchors {
                left: col.left
                right: col.right
            }
            onTextChanged: {
                if (parseInt(text) !== encryptionSelector.model.get(encryptionSelector.selectedIndex).port) {
                    smtpPortWarnLabel.text = "This port is nonstandard. The default port is %1".arg(
                                encryptionSelector.model.get(encryptionSelector.selectedIndex).port)
                } else {
                    smtpPortWarnLabel.text = ""
                }
            }
        }

        Item {
            id: authItem
            width: parent.width
            height: smtpPortInput.height
            property alias checked: authCheckBox.checked
            Label {
                text: i18n.tr("Authenticate")
                anchors.left: parent.left
                anchors.verticalCenter: parent.verticalCenter
            }
            CheckBox {
                id: authCheckBox
                anchors.right: parent.right
            }
        }

        Label {
            id: smtpUsernameLabel
            visible: authCheckBox.checked
            property bool valid: !!smtpUserNameInput.text
            text: valid ? i18n.tr("Username") : i18n.tr("Username required")
            color: valid ? "#888888" : "red"
        }
        TextField {
            id: smtpUserNameInput
            visible: smtpUsernameLabel.visible
            anchors {
                left: col.left;
                right: col.right;
            }
            inputMethodHints: Qt.ImhNoAutoUppercase | Qt.ImhEmailCharactersOnly | Qt.ImhNoPredictiveText
        }

        Label {
            id: smtpPasswordLabel
            visible: smtpUsernameLabel.visible
            text: i18n.tr("Password")
        }
        TextField {
            id: smtpPasswordInput
            visible: smtpPasswordLabel.visible
            inputMethodHints: Qt.ImhHiddenText | Qt.ImhNoAutoUppercase | Qt.ImhNoPredictiveText
            echoMode: TextInput.Password
            text: smtpPasswordWatcher.password
            anchors {
                left: parent.left;
                right: parent.right;
            }
        }
        Label {
            id: passwordWarning
            anchors {
                left: parent.left
                right: parent.right
            }
            visible: smtpPasswordInput.text.length > 0
            wrapMode: TextEdit.Wrap
            text: i18n.tr("This password will be saved in encrypted storage. " +
                          "If you do not enter password here, Dekko will prompt for one when needed.")
        }

        Label {
            id: smtpSendMail
            visible: false
            text: i18n.tr("Sendmail executable")
        }
        TextField {
            id: smtpSendMailInput
            visible: smtpSendMail.visible
            anchors {
                left: col.left;
                right: col.right;
            }

        }

        Item {
            id: saveOutgoingItem
            width: parent.width
            height: smtpPortInput.height
            property alias checked: outgoingCheckbox.checked
            Label {
                text: i18n.tr("Save outgoing")
                anchors.left: parent.left
                anchors.verticalCenter: parent.verticalCenter
            }
            CheckBox {
                id: outgoingCheckbox
                anchors.right: parent.right
            }
        }

        Label {
            id: smtpSentLocation
            visible: saveOutgoingItem.checked
            text: i18n.tr("Sent folder name")
        }
        TextField {
            id: smtpSentLocationInput
            visible: smtpSentLocation.visible
            anchors {
                left: parent.left;
                right: parent.right;
            }
        }

        Item {
            id: burlItem
            width: parent.width
            height: smtpPortInput.height
            visible: saveOutgoingItem.checked
            Label {
                text: i18n.tr("Send with BURL")
                anchors.left: parent.left
                anchors.verticalCenter: parent.verticalCenter
            }
            CheckBox {
                id: burlCheckbox
                anchors.right: parent.right
            }
        }
    }


    Component.onCompleted: {
        // we have to do this here before setting the selectors index otherwise
        // we can't guarentee the model is ready before we set the index.
        encryptionMethodModel.append({ "description": i18n.tr("No encryption"), "method": SubmissionMethod.SMTP, "port": 587 })
        encryptionMethodModel.append({ "description": i18n.tr("Use encryption (STARTTLS)"), "method": SubmissionMethod.SMTP_STARTTLS, "port": 587 })
        encryptionMethodModel.append({ "description": i18n.tr("Force encryption (SSL/TLS)"), "method": SubmissionMethod.SSMTP, "port": 465 })
        if (account.submissionMethod) {
            switch (account.submissionMethod){
            case SubmissionMethod.SMTP:
            case SubmissionMethod.SMTP_STARTTLS:
            case SubmissionMethod.SSMTP:
                state = "NETWORK_SETTINGS"
                break
            case SubmissionMethod.SENDMAIL:
                state = "SENDMAIL_SETTINGS"
                break
            case SubmissionMethod.IMAP_SENDMAIL:
                state = "IMAP_SENDMAIL_SETTINGS"
                break
            }
        }
        console.log("SUBMISSION METHOD: " + account.submissionMethod)
        switch (account.submissionMethod){
        case SubmissionMethod.SMTP_STARTTLS:
            smtpEncryptionModelIndex = 1
            break
        case SubmissionMethod.SSMTP:
            smtpEncryptionModelIndex = 2
            break
        default:
            smtpEncryptionModelIndex = 0
        }
        smtpPasswordWatcher.reloadPassword(currentAccount.accountName, "smtp")
        smtpPathToSendmail = account.submissionPathToSendmail
        smtpAuthenticate = account.submissionAuthenticateEnabled
        smtpBurl = account.submissionUseBurl
        smtpUserName = account.submissionUsername
        smtpServerInput.text = account.submissionServer
        smtpPort = account.submissionPort !== 0 ?
                    account.submissionPort :
                    encryptionSelector.model.get(encryptionSelector.selectedIndex).port
        smtpSentLocationInput.text = account.submissionSentMailboxName
        smtpSaveToImap = account.submissionSaveToImap
    }

    Connections {
        target: dekko
        onAutoConfigComplete: {
            smtpConnectionModelIndex = 0; //Network
            smtpServerInput.text = config.smtpHost;
            for (var i = 0; i < encryptionMethodModel.count; i++) {
                if (config.smtpMethod === encryptionMethodModel.get(i).method) {
                    encryptionSelector.selectedIndex = i;
                }
            }
            smtpPort = config.smtpPort;
            //set authentication to true as default
            smtpAuthenticate = true;
            smtpUserName = username;
            smtpPassword = password;
        }
        onManualConfig: {
            smtpConnectionModelIndex = 0; //Network
            smtpAuthenticate = true;
            smtpUserName = username;
            smtpPassword = password;
            smtpServer = "";
            smtpEncryptionModelIndex = 0; //SMTP
        }
    }
}
