#include "Imap/Network/MsgPartNetAccessManager.h"
#include "Imap/Model/NetworkWatcher.h"
#ifndef UIUTILS_PARTWIDGETFACTORY
#define UIUTILS_PARTWIDGETFACTORY

namespace UiUtils {

class PartWidgetFactory : public QObject
{
    Q_OBJECT
    enum { ExpensiveFetchThreshold = 50*1024 };
public:
    typedef enum {
        PART_IGNORE_DISPOSITION_ATTACHMENT = 1 << 0, /**< @short Don't wrap the part in an AttachmentView */
        PART_IGNORE_CLICKTHROUGH = 1 << 1, /**< @short Ignore any heuristics which lead to wrapping in an LoadablePartWidget with a clickthrough */
        PART_IGNORE_LOAD_ON_SHOW = 1 << 2, /**< @short Ignore wrapping in a LoadablePartWidget set up to load on first show event */
        PART_IS_HIDDEN = 1 << 3, /**< @short Request wrapping this part in a LoadablePartWidget */
        PART_PREFER_PLAINTEXT_OVER_HTML = 1 << 4, /**< @short The user's preferences indicate that a text/plain part shall be shown instead of a text/html if available */
        PART_DONT_REGISTER_AS_SECTION = 1 << 5
    } PartLoadingFlag;
    /** @short Which of these options shall be propagated to child widgets when embedding them, and which shall be filtered? */
    enum {
        MASK_PROPAGATE_WHEN_EMBEDDING =
            PART_IS_HIDDEN | PART_PREFER_PLAINTEXT_OVER_HTML
    };
    typedef QFlags<PartLoadingFlag> PartLoadingOptions;

    PartWidgetFactory(Imap::Network::MsgPartNetAccessManager *manager = 0);

    Q_INVOKABLE QObject *create(const QModelIndex &partIndex, QObject *messageView,
                                 int recursionDepth, bool preferPlainText);
    Q_INVOKABLE void setNetAccessManager(Imap::Network::MsgPartNetAccessManager *manager);
    Q_INVOKABLE void setNetWatcher(Imap::Mailbox::NetworkWatcher *netWatcher);

    QObject *create(const QModelIndex &partIndex, QObject *messageView,
                    int recursionDepth, const PartLoadingOptions loadingMode);

    void setNetworkWatcher(Imap::Mailbox::NetworkWatcher *netWatcher);

    QObject *newLabelWidget(QString);
    QObject *newLoadablePartWidget(QObject *parent, Imap::Network::MsgPartNetAccessManager *manager,
                                    const QModelIndex &part, QObject *messageView,
                                    UiUtils::PartWidgetFactory *factory, int recursionDepth,
                                    const PartLoadingOptions loadingMode);
    void hideWidget(QObject *widget);
    QObject *newAttachmentView(QObject *parent, Imap::Network::MsgPartNetAccessManager *manager,
                               const QModelIndex &partIndex, QObject *messageView,
                               QObject *contentWidget);
    QObject *newMultipartAlternativeWidget(QObject *parent, UiUtils::PartWidgetFactory *factory,
                                           const QModelIndex &partIndex, QObject *messageView,
                                           const int recursionDepth, const PartLoadingOptions options);
    QObject *newMultipartSignedWidget(QObject *parent, UiUtils::PartWidgetFactory *factory,
                                      const QModelIndex &partIndex,
                                           const int recursionDepth, const PartLoadingOptions options);
    QObject *newGenericMultipartWidget(QObject *parent, UiUtils::PartWidgetFactory *factory,
                                       const QModelIndex &partIndex, QObject *messageView,
                                       const int recursionDepth, const PartLoadingOptions options);
    QObject *newMessage822Widget(QObject *parent, UiUtils::PartWidgetFactory *factory,
                                 const QModelIndex &partIndex,
                                 const int recursionDepth, const PartLoadingOptions options);
    QObject *newSimplePartWidget(QObject *parent, Imap::Network::MsgPartNetAccessManager *manager,
                                 const QModelIndex &partIndex, QObject *messageView, bool dontRegisterAsSection);

private:
    Imap::Network::MsgPartNetAccessManager *m_manager;
    Imap::Mailbox::NetworkWatcher *m_netWatcher;

    /** @short Unset flags which only make sense for one level of nesting */
    PartLoadingOptions filteredForEmbedding(const UiUtils::PartWidgetFactory::PartLoadingOptions options);
};

}

#endif //UIUTILS_PARTWIDGETFACTORY
