/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of Dekko email client for Ubuntu Devices/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef SENDERIDENTITYMODEL_H
#define SENDERIDENTITYMODEL_H

#include <QObject>
#include <QAbstractListModel>
#include <QModelIndex>
#include <QSettings>
#include "SenderIdentity.h"

namespace Dekko
{
namespace Accounts
{
class SenderIdentityModel : public QAbstractListModel
{
    Q_OBJECT
    Q_PROPERTY(int count READ rowCount NOTIFY countChanged)
public:
    explicit SenderIdentityModel(QObject *parent = 0);

    typedef QList<SenderIdentity *> IdentitiesList;

    enum Roles {
        IdentityNameRole = Qt::UserRole + 1,
        IdentityEmailRole,
        IdentityOrganizationRole,
        IdentitySignatureRole,
        IdentityUseDefaultSignature,
        IdentityRecipientStringRole
    };
    Q_INVOKABLE void reloadIdentities(QSettings *settings);
    Q_INVOKABLE void saveIdentites(QSettings *settings);

    QHash<int, QByteArray> roleNames() const override;
    QVariant data(const QModelIndex &index, int role) const;
    int rowCount(const QModelIndex &parent=QModelIndex()) const;
    QModelIndex indexFromItem(SenderIdentity *identity) const;
    Q_INVOKABLE bool removeRow(int row, const QModelIndex &index = QModelIndex());
    Q_INVOKABLE bool removeRows(int row, int count, const QModelIndex &index = QModelIndex());
    Q_INVOKABLE QVariant get(int index);
    bool setData(const QModelIndex &index, const QVariant &value, int role);

signals:
    void countChanged();

public slots:
    void appendRow(SenderIdentity *identity);
    void appendRows(QList<SenderIdentity *> &identities);
    void insertRow(int row, SenderIdentity *identity);
    void clear();

private slots:
    void updateIdentity();

private:
    IdentitiesList m_identities;

    void migrateFromTempIdentities(QSettings *settings);

};
}
}
#endif // SENDERIDENTITYMODEL_H
