#include "AttachmentView.h"
#include <QQmlComponent>
#include <QMimeType>
#include <QMimeDatabase>
#include <QQmlEngine>
#include <QDir>
#include "app/Utils/Path.h"
#include "Imap/Model/ItemRoles.h"
#include "Imap/Network/FileDownloadManager.h"
#include "UiUtils/Formatting.h"
namespace Dekko {

namespace PartWidget {
AttachmentView::AttachmentView(QQuickItem *messageView,
    Imap::Network::MsgPartNetAccessManager *manager,
    const QModelIndex &partIndex, QQuickItem *contentWidget, QQmlEngine *engine)
    : m_messageView(messageView), m_manager(manager), m_partIndex(partIndex),
      m_contentWidget(contentWidget), m_engine(engine), m_tmpFile(0),
      m_item(0)
{
    setParent(messageView);
}

QQuickItem *AttachmentView::createItem()
{
    QQmlComponent component(m_engine, Dekko::findQmlFile(
                                "qml/MessageView/AttachmentView.qml"));

    m_item = qobject_cast<QQuickItem*>(
                component.create(m_engine->contextForObject(m_messageView)));
    m_item->setParent(m_messageView);

    m_item->setProperty("fileName",
                      m_partIndex.data(Imap::Mailbox::RolePartFileName));

    QString mimeDescription = m_partIndex.data(Imap::Mailbox::RolePartMimeType).toString();
    QMimeType mimeType = QMimeDatabase().mimeTypeForName(mimeDescription);
    if (mimeType.isValid() && !mimeType.isDefault()) {
        mimeDescription = mimeType.comment();
    }
    m_item->setProperty("mimeDescription", mimeDescription);

    m_item->setProperty("fileSize", UiUtils::Formatting::prettySize(
            m_partIndex.data(Imap::Mailbox::RolePartOctets).
            toUInt(), UiUtils::Formatting::BytesSuffix::WITH_BYTES_SUFFIX));

    if (m_contentWidget != NULL) {
        m_item->setProperty("previewWidget",
                                QVariant::fromValue(m_contentWidget));
        m_contentWidget->setParentItem(m_item);
    }

    connect(m_item, SIGNAL(open()), this, SLOT(onOpenAttachment()));
    connect(m_item, SIGNAL(openComplete()), this, SLOT(onOpenComplete()));
    return m_item;
}

void AttachmentView::onOpenAttachment()
{
//    m_openAttachment->setEnabled(false);

    Imap::Network::FileDownloadManager *fileDownloadManager =
        new Imap::Network::FileDownloadManager(this, m_manager, m_partIndex);
    connect(fileDownloadManager, SIGNAL(fileNameRequested(QString*)), this, SLOT(slotFileNameRequestedOnOpen(QString*)));
//    connect(fileDownloadManager, SIGNAL(transferError(QString)), m_messageView, SIGNAL(transferError(QString)));
    connect(fileDownloadManager, SIGNAL(transferError(QString)), this, SLOT(onOpenFailed(QString)));
    connect(fileDownloadManager, SIGNAL(transferError(QString)), fileDownloadManager, SLOT(deleteLater()));
    // we aren't connecting to cancelled() as it cannot really happen -- the filename is never empty
    connect(fileDownloadManager, SIGNAL(succeeded()), this, SLOT(openDownloadedAttachment()));
    connect(fileDownloadManager, SIGNAL(succeeded()), fileDownloadManager, SLOT(deleteLater()));
    fileDownloadManager->downloadPart();
}

void AttachmentView::openDownloadedAttachment()
{
    Q_ASSERT(m_tmpFile);

    // Make sure that the file is read-only so that the launched application does not attempt to modify it
    m_tmpFile->setPermissions(QFile::ReadOwner);

    QMetaObject::invokeMethod(m_item, "openDownloadComplete",
        Q_ARG(QVariant, QVariant::fromValue(QUrl::fromLocalFile(m_tmpFile->fileName()))));
}

void AttachmentView::slotFileNameRequestedOnOpen(QString * fileName)
{
    Q_ASSERT(!m_tmpFile);
    m_tmpFile = new QTemporaryFile(QDir::tempPath() +
                                   QLatin1String("/trojita-attachment-XXXXXX-") +
                                   fileName->replace(QLatin1Char('/'), QLatin1Char('_')));
    m_tmpFile->open();
    *fileName = m_tmpFile->fileName();
}

void AttachmentView::onOpenFailed(QString message)
{
    QMetaObject::invokeMethod(m_messageView, "alertError",
                              Q_ARG(QVariant, QVariant::fromValue(message)));
    delete m_tmpFile;
    m_tmpFile = 0;
//    m_openAttachment->setEnabled(true);
}

void AttachmentView::onOpenComplete() {
    delete m_tmpFile;
    m_tmpFile = 0;
//        m_openAttachment->setEnabled(true);
}
}

}
