#include "LoadablePartWidget.h"
#include <QQmlEngine>
#include "app/Utils/Path.h"
#include "UiUtils/PartWidgetFactory.h"


namespace Dekko {

namespace PartWidget {
LoadablePartWidget::LoadablePartWidget(
        QQuickItem *messageView, const QModelIndex &partIndex,
        UiUtils::PartWidgetFactory *factory, int recursionDepth,
        const UiUtils::PartWidgetFactory::PartLoadingOptions loadingOptions,
        QQmlEngine *qmlEngine)
    : m_messageView(messageView), m_partIndex(partIndex),
      m_factory(factory), m_recursionDepth(recursionDepth),
      m_loadingOptions(loadingOptions), m_qmlEngine(qmlEngine)
{
    setParent(messageView);
}

QQuickItem * LoadablePartWidget::createItem() {
    QQmlComponent component(m_qmlEngine, Dekko::findQmlFile("qml/MessageView/LoadablePartWidget.qml"));

    m_item = qobject_cast<QQuickItem *>(component.create(m_qmlEngine->contextForObject(m_messageView)));
    connect(m_item, SIGNAL(loadContent()), this, SLOT(handleLoadContent()));
    return m_item;
}

void LoadablePartWidget::handleLoadContent() {
    qobject_cast<QQuickItem *>(
                m_factory->create(m_partIndex, m_messageView,
                                  m_recursionDepth + 1,
                                  (m_loadingOptions
                                   | UiUtils::PartWidgetFactory::PART_IGNORE_CLICKTHROUGH
                                   | UiUtils::PartWidgetFactory::PART_IGNORE_LOAD_ON_SHOW)
                                  ^ UiUtils::PartWidgetFactory::PART_IS_HIDDEN))
            ->setParentItem(m_item);
}

}
}
